@@ -91,7 +91,7 @@ sub process
 
     my $oHtmlBuilder = new BackRestDoc::Html::DocHtmlBuilder(
         $self->{oManifest}->variableReplace('{[project]}' . (defined($self->{oManifest}->variableGet('project-tagline')) ?
-            $self->{oManifest}->variableGet('project-tagline') : '')),
+            ' - ' . $self->{oManifest}->variableGet('project-tagline') : '')),
         $self->{oManifest}->variableReplace($strTitle . (defined($strSubTitle) ? " - ${strSubTitle}" : '')),
         $self->{oManifest}->variableGet('project-favicon'),
         $self->{oManifest}->variableGet('project-logo'),
@@ -135,9 +135,13 @@ sub process
             {
                 my $oRenderOut = $self->{oManifest}->renderOutGet(RENDER_TYPE_HTML, $strRenderOutKey);
 
-                $oMenuBody->
-                    addNew(HTML_DIV, 'menu')->
-                        addNew(HTML_A, 'menu-link', {strContent => $$oRenderOut{menu}, strRef => "${strRenderOutKey}.html"});
+                if (defined($$oRenderOut{menu}))
+                {
+                    $oMenuBody->addNew(HTML_DIV, 'menu')->addNew(
+                        HTML_A, 'menu-link',
+                        {strContent => $$oRenderOut{menu},
+                            strRef => $strRenderOutKey eq 'index' ? '{[project-url-root]}' : "${strRenderOutKey}.html"});
+                }
             }
         }
     }
@@ -433,7 +437,7 @@ sub sectionProcess
 
                     $oHeaderRowElement->addNew(
                         HTML_TH,
-                        "table-header-${strAlign}" . ($bFill ? ",table-header-fill" : ""),
+                        "table-header-${strAlign}" . ($bFill ? " table-header-fill" : ""),
                         {strContent => $self->processText($oColumn->textGet())});
                 }
             }
