@@ -6,6 +6,7 @@ Archive Common
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "command/archive/common.h"
 #include "common/debug.h"
@@ -69,20 +70,20 @@ archiveAsyncStatus(ArchiveMode archiveMode, const String *walSegment, bool confe
         const String *spoolQueue = archiveAsyncSpoolQueue(archiveMode);
 
         String *okFile = strNewFmt("%s" STATUS_EXT_OK, strPtr(walSegment));
-        bool okFileExists = storageExistsNP(storageSpool(), strNewFmt("%s/%s", strPtr(spoolQueue), strPtr(okFile)));
+        bool okFileExists = storageExistsP(storageSpool(), strNewFmt("%s/%s", strPtr(spoolQueue), strPtr(okFile)));
 
         // If the ok file does not exist then check to see if a file-specific or global error exists
         if (!okFileExists)
         {
             // Check for a file-specific error first
             errorFile = strNewFmt("%s" STATUS_EXT_ERROR, strPtr(walSegment));
-            errorFileExists = storageExistsNP(storageSpool(), strNewFmt("%s/%s", strPtr(spoolQueue), strPtr(errorFile)));
+            errorFileExists = storageExistsP(storageSpool(), strNewFmt("%s/%s", strPtr(spoolQueue), strPtr(errorFile)));
 
             // If that doesn't exist then check for a global error
             if (!errorFileExists)
             {
                 errorFile = STATUS_FILE_GLOBAL_ERROR_STR;
-                errorFileExists = storageExistsNP(storageSpool(), strNewFmt("%s/%s", strPtr(spoolQueue), strPtr(errorFile)));
+                errorFileExists = storageExistsP(storageSpool(), strNewFmt("%s/%s", strPtr(spoolQueue), strPtr(errorFile)));
             }
         }
 
@@ -93,7 +94,7 @@ archiveAsyncStatus(ArchiveMode archiveMode, const String *walSegment, bool confe
             const String *statusFile = okFileExists ? okFile: errorFile;
 
             String *content = strNewBuf(
-                storageGetNP(storageNewReadNP(storageSpool(), strNewFmt("%s/%s", strPtr(spoolQueue), strPtr(statusFile)))));
+                storageGetP(storageNewReadP(storageSpool(), strNewFmt("%s/%s", strPtr(spoolQueue), strPtr(statusFile)))));
 
             // Get the code and message if the file has content
             int code = 0;
@@ -172,8 +173,8 @@ archiveAsyncStatusErrorWrite(ArchiveMode archiveMode, const String *walSegment,
     {
         const String *errorFile = walSegment == NULL ? STATUS_FILE_GLOBAL_STR : walSegment;
 
-        storagePutNP(
-            storageNewWriteNP(
+        storagePutP(
+            storageNewWriteP(
                 storageSpoolWrite(),
                 strNewFmt("%s/%s" STATUS_EXT_ERROR, strPtr(archiveAsyncSpoolQueue(archiveMode)), strPtr(errorFile))),
             BUFSTR(strNewFmt("%d\n%s", code, strPtr(message))));
@@ -200,8 +201,8 @@ archiveAsyncStatusOkWrite(ArchiveMode archiveMode, const String *walSegment, con
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Write file
-        storagePutNP(
-            storageNewWriteNP(
+        storagePutP(
+            storageNewWriteP(
                 storageSpoolWrite(),
                 strNewFmt("%s/%s" STATUS_EXT_OK, strPtr(archiveAsyncSpoolQueue(archiveMode)), strPtr(walSegment))),
             warning == NULL ? NULL : BUFSTR(strNewFmt("0\n%s", strPtr(warning))));
@@ -246,16 +247,37 @@ walPath(const String *walFile, const String *pgPath, const String *command)
 
     if (!strBeginsWithZ(walFile, "/"))
     {
+        // Error if walFile has a relative path and pgPath is not set
         if (pgPath == NULL)
         {
             THROW_FMT(
                 OptionRequiredError,
                 "option '" CFGOPT_PG1_PATH "' must be specified when relative wal paths are used\n"
-                    "HINT: Is %%f passed to %s instead of %%p?\n"
+                    "HINT: is %%f passed to %s instead of %%p?\n"
                     "HINT: PostgreSQL may pass relative paths even with %%p depending on the environment.",
                 strPtr(command));
         }
 
+        // Get the working directory
+        char currentWorkDir[4096];
+        THROW_ON_SYS_ERROR(getcwd(currentWorkDir, sizeof(currentWorkDir)) == NULL, FormatError, "unable to get cwd");
+
+        // Check if the working directory is the same as pgPath
+        if (!strEqZ(pgPath, currentWorkDir))
+        {
+            // If not we'll change the working directory to pgPath and see if that equals the working directory we got called with
+            THROW_ON_SYS_ERROR_FMT(chdir(strPtr(pgPath)) != 0, PathMissingError, "unable to chdir() to '%s'", strPtr(pgPath));
+
+            // Get the new working directory
+            char newWorkDir[4096];
+            THROW_ON_SYS_ERROR(getcwd(newWorkDir, sizeof(newWorkDir)) == NULL, FormatError, "unable to get cwd");
+
+            // Error if the new working directory is not equal to the original current working directory.  This means that
+            // PostgreSQL and pgBackrest have a different idea about where the PostgreSQL data directory is located.
+            if (strcmp(currentWorkDir, newWorkDir) != 0)
+                THROW_FMT(AssertError, "working path '%s' is not the same path as '%s'", currentWorkDir, strPtr(pgPath));
+        }
+
         result = strNewFmt("%s/%s", strPtr(pgPath), strPtr(walFile));
     }
     else
@@ -339,16 +361,28 @@ walSegmentFind(const Storage *storage, const String *archiveId, const String *wa
                         strPtr(walSegment), strPtr(strLstJoin(strLstSort(list, sortOrderAsc), ", ")));
                 }
 
-                // Copy file name of WAL segment found into the calling context
-                memContextSwitch(MEM_CONTEXT_OLD());
-                result = strDup(strLstGet(list, 0));
-                memContextSwitch(MEM_CONTEXT_TEMP());
+                // Copy file name of WAL segment found into the prior context
+                MEM_CONTEXT_PRIOR_BEGIN()
+                {
+                    result = strDup(strLstGet(list, 0));
+                }
+                MEM_CONTEXT_PRIOR_END();
             }
         }
         while (result == NULL && wait != NULL && waitMore(wait));
     }
     MEM_CONTEXT_TEMP_END();
 
+    if (result == NULL && timeout != 0)
+    {
+        THROW_FMT(
+            ArchiveTimeoutError,
+            "WAL segment %s was not archived before the %" PRIu64 "ms timeout\n"
+                "HINT: check the archive_command to ensure that all options are correct (especially --stanza).\n"
+                "HINT: check the PostgreSQL server log for errors.",
+            strPtr(walSegment), timeout);
+    }
+
     FUNCTION_LOG_RETURN(STRING, result);
 }
 
@@ -419,7 +453,8 @@ walSegmentRange(const String *walSegmentBegin, size_t walSegmentSize, unsigned i
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        result = strLstAdd(strLstNew(), walSegmentBegin);
+        result = strLstNew();
+        strLstAdd(result, walSegmentBegin);
 
         if (range > 1)
         {
@@ -436,7 +471,7 @@ walSegmentRange(const String *walSegmentBegin, size_t walSegmentSize, unsigned i
             }
         }
 
-        strLstMove(result, MEM_CONTEXT_OLD());
+        strLstMove(result, memContextPrior());
     }
     MEM_CONTEXT_TEMP_END();
 
