@@ -32,6 +32,7 @@ STRING_EXTERN(HTTP_HEADER_CONTENT_MD5_STR,                          HTTP_HEADER_
 #define HTTP_HEADER_TRANSFER_ENCODING                               "transfer-encoding"
     STRING_STATIC(HTTP_HEADER_TRANSFER_ENCODING_STR,                HTTP_HEADER_TRANSFER_ENCODING);
 STRING_EXTERN(HTTP_HEADER_ETAG_STR,                                 HTTP_HEADER_ETAG);
+STRING_EXTERN(HTTP_HEADER_LAST_MODIFIED_STR,                        HTTP_HEADER_LAST_MODIFIED);
 
 #define HTTP_VALUE_CONNECTION_CLOSE                                 "close"
     STRING_STATIC(HTTP_VALUE_CONNECTION_CLOSE_STR,                  HTTP_VALUE_CONNECTION_CLOSE);
@@ -61,7 +62,7 @@ struct HttpClient
     String *responseMessage;                                        // Response message e.g. (OK, Not Found)
     HttpHeader *responseHeader;                                     // Response headers
 
-    bool contentChunked;                                            // Is the reponse content chunked?
+    bool contentChunked;                                            // Is the response content chunked?
     uint64_t contentSize;                                           // Content size (ignored for chunked)
     uint64_t contentRemaining;                                      // Content remaining (per chunk if chunked)
     bool closeOnContentEof;                                         // Will server close after content is sent?
@@ -193,12 +194,14 @@ httpClientNew(
 
     MEM_CONTEXT_NEW_BEGIN("HttpClient")
     {
-        // Allocate state and set context
         this = memNew(sizeof(HttpClient));
-        this->memContext = MEM_CONTEXT_NEW();
 
-        this->timeout = timeout;
-        this->tls = tlsClientNew(host, port, timeout, verifyPeer, caFile, caPath);
+        *this = (HttpClient)
+        {
+            .memContext = MEM_CONTEXT_NEW(),
+            .timeout = timeout,
+            .tls = tlsClientNew(host, port, timeout, verifyPeer, caFile, caPath),
+        };
 
         httpClientStatLocal.object++;
     }
@@ -389,7 +392,7 @@ httpClientRequest(
                 bool contentExists = (this->contentChunked || this->contentSize > 0) && !strEq(verb, HTTP_VERB_HEAD_STR);
                 this->contentEof = !contentExists;
 
-                // If all content should be returned from this function then read the buffer.  Also read the reponse if there has
+                // If all content should be returned from this function then read the buffer.  Also read the response if there has
                 // been an error.
                 if (returnContent || !httpClientResponseCodeOk(this))
                 {
@@ -435,7 +438,7 @@ httpClientRequest(
                 // Retry if wait time has not expired
                 if (wait != NULL && waitMore(wait))
                 {
-                    LOG_DEBUG("retry %s: %s", errorTypeName(errorType()), errorMessage());
+                    LOG_DEBUG_FMT("retry %s: %s", errorTypeName(errorType()), errorMessage());
                     retry = true;
 
                     httpClientStatLocal.retry++;
@@ -451,7 +454,7 @@ httpClientRequest(
             RETHROW();
 
         // Move the result buffer (if any) to the parent context
-        bufMove(result, MEM_CONTEXT_OLD());
+        bufMove(result, memContextPrior());
 
         httpClientStatLocal.request++;
     }
@@ -570,7 +573,7 @@ httpClientResponseCodeOk(const HttpClient *this)
 Get the response headers
 ***********************************************************************************************************************************/
 const HttpHeader *
-httpClientReponseHeader(const HttpClient *this)
+httpClientResponseHeader(const HttpClient *this)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(HTTP_CLIENT, this);
