@@ -20,6 +20,9 @@ Constants
 STRING_EXTERN(PROTOCOL_SERVICE_LOCAL_STR,                           PROTOCOL_SERVICE_LOCAL);
 STRING_EXTERN(PROTOCOL_SERVICE_REMOTE_STR,                          PROTOCOL_SERVICE_REMOTE);
 
+STRING_STATIC(PROTOCOL_REMOTE_TYPE_PG_STR,                          PROTOCOL_REMOTE_TYPE_PG);
+STRING_STATIC(PROTOCOL_REMOTE_TYPE_REPO_STR,                        PROTOCOL_REMOTE_TYPE_REPO);
+
 /***********************************************************************************************************************************
 Local variables
 ***********************************************************************************************************************************/
@@ -33,10 +36,10 @@ static struct
 {
     MemContext *memContext;                                         // Mem context for protocol helper
 
-    unsigned int clientRemoteSize;                                 // Remote clients
+    unsigned int clientRemoteSize;                                  // Remote clients
     ProtocolHelperClient *clientRemote;
 
-    unsigned int clientLocalSize;                                  // Local clients
+    unsigned int clientLocalSize;                                   // Local clients
     ProtocolHelperClient *clientLocal;
 } protocolHelper;
 
@@ -52,7 +55,11 @@ protocolHelperInit(void)
         // Create a mem context to store protocol objects
         MEM_CONTEXT_BEGIN(memContextTop())
         {
-            protocolHelper.memContext = memContextNew("ProtocolHelper");
+            MEM_CONTEXT_NEW_BEGIN("ProtocolHelper")
+            {
+                protocolHelper.memContext = MEM_CONTEXT_NEW();
+            }
+            MEM_CONTEXT_NEW_END();
         }
         MEM_CONTEXT_END();
     }
@@ -68,6 +75,20 @@ repoIsLocal(void)
     FUNCTION_TEST_RETURN(!cfgOptionTest(cfgOptRepoHost));
 }
 
+/***********************************************************************************************************************************
+Error if the repository is not local
+***********************************************************************************************************************************/
+void
+repoIsLocalVerify(void)
+{
+    FUNCTION_TEST_VOID();
+
+    if (!repoIsLocal())
+        THROW_FMT(HostInvalidError, "%s command must be run on the repository host", cfgCommandName(cfgCommand()));
+
+    FUNCTION_TEST_RETURN_VOID();
+}
+
 /***********************************************************************************************************************************
 Is pg local?
 ***********************************************************************************************************************************/
@@ -78,6 +99,8 @@ pgIsLocal(unsigned int hostId)
         FUNCTION_LOG_PARAM(UINT, hostId);
     FUNCTION_LOG_END();
 
+    ASSERT(hostId > 0);
+
     FUNCTION_LOG_RETURN(BOOL, !cfgOptionTest(cfgOptPgHost + hostId - 1));
 }
 
@@ -85,13 +108,16 @@ pgIsLocal(unsigned int hostId)
 Get the command line required for local protocol execution
 ***********************************************************************************************************************************/
 static StringList *
-protocolLocalParam(ProtocolStorageType protocolStorageType, unsigned int protocolId)
+protocolLocalParam(ProtocolStorageType protocolStorageType, unsigned int hostId, unsigned int protocolId)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(ENUM, protocolStorageType);
+        FUNCTION_LOG_PARAM(UINT, hostId);
         FUNCTION_LOG_PARAM(UINT, protocolId);
     FUNCTION_LOG_END();
 
+    ASSERT(hostId > 0);
+
     StringList *result = NULL;
 
     MEM_CONTEXT_TEMP_BEGIN()
@@ -99,17 +125,14 @@ protocolLocalParam(ProtocolStorageType protocolStorageType, unsigned int protoco
         // Option replacements
         KeyValue *optionReplace = kvNew();
 
-        // Add the command option
-        kvPut(optionReplace, VARSTR(CFGOPT_COMMAND_STR), VARSTRZ(cfgCommandName(cfgCommand())));
-
         // Add the process id -- used when more than one process will be called
-        kvPut(optionReplace, VARSTR(CFGOPT_PROCESS_STR), VARINT((int)protocolId));
+        kvPut(optionReplace, VARSTR(CFGOPT_PROCESS_STR), VARUINT(protocolId));
 
-        // Add the host id -- for now this is hard-coded to 1
-        kvPut(optionReplace, VARSTR(CFGOPT_HOST_ID_STR), VARINT(1));
+        // Add the host id
+        kvPut(optionReplace, VARSTR(CFGOPT_HOST_ID_STR), VARUINT(hostId));
 
-        // Add the type
-        kvPut(optionReplace, VARSTR(CFGOPT_TYPE_STR), VARSTRDEF("backup"));
+        // Add the remote type
+        kvPut(optionReplace, VARSTR(CFGOPT_REMOTE_TYPE_STR), VARSTR(protocolStorageTypeStr(protocolStorageType)));
 
         // Only enable file logging on the local when requested
         kvPut(
@@ -119,7 +142,10 @@ protocolLocalParam(ProtocolStorageType protocolStorageType, unsigned int protoco
         // Always output errors on stderr for debugging purposes
         kvPut(optionReplace, VARSTR(CFGOPT_LOG_LEVEL_STDERR_STR), VARSTRDEF("error"));
 
-        result = strLstMove(cfgExecParam(cfgCmdLocal, optionReplace), MEM_CONTEXT_OLD());
+        // Disable output to stdout since it is used by the protocol
+        kvPut(optionReplace, VARSTR(CFGOPT_LOG_LEVEL_CONSOLE_STR), VARSTRDEF("off"));
+
+        result = strLstMove(cfgExecParam(cfgCommand(), cfgCmdRoleLocal, optionReplace, true, false), memContextPrior());
     }
     MEM_CONTEXT_TEMP_END();
 
@@ -130,13 +156,16 @@ protocolLocalParam(ProtocolStorageType protocolStorageType, unsigned int protoco
 Get the local protocol client
 ***********************************************************************************************************************************/
 ProtocolClient *
-protocolLocalGet(ProtocolStorageType protocolStorageType, unsigned int protocolId)
+protocolLocalGet(ProtocolStorageType protocolStorageType, unsigned int hostId, unsigned int protocolId)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(ENUM, protocolStorageType);
+        FUNCTION_LOG_PARAM(UINT, hostId);
         FUNCTION_LOG_PARAM(UINT, protocolId);
     FUNCTION_LOG_END();
 
+    ASSERT(hostId > 0);
+
     protocolHelperInit();
 
     // Allocate the client cache
@@ -144,9 +173,11 @@ protocolLocalGet(ProtocolStorageType protocolStorageType, unsigned int protocolI
     {
         MEM_CONTEXT_BEGIN(protocolHelper.memContext)
         {
-            protocolHelper.clientLocalSize = cfgOptionUInt(cfgOptProcessMax);
-            protocolHelper.clientLocal = (ProtocolHelperClient *)memNew(
-                protocolHelper.clientLocalSize * sizeof(ProtocolHelperClient));
+            protocolHelper.clientLocalSize = cfgOptionUInt(cfgOptProcessMax) + 1;
+            protocolHelper.clientLocal = memNew(protocolHelper.clientLocalSize * sizeof(ProtocolHelperClient));
+
+            for (unsigned int clientIdx = 0; clientIdx < protocolHelper.clientLocalSize; clientIdx++)
+                protocolHelper.clientLocal[clientIdx] = (ProtocolHelperClient){.exec = NULL};
         }
         MEM_CONTEXT_END();
     }
@@ -162,7 +193,7 @@ protocolLocalGet(ProtocolStorageType protocolStorageType, unsigned int protocolI
         {
             // Execute the protocol command
             protocolHelperClient->exec = execNew(
-                cfgExe(), protocolLocalParam(protocolStorageType, protocolId),
+                cfgExe(), protocolLocalParam(protocolStorageType, hostId, protocolId),
                 strNewFmt(PROTOCOL_SERVICE_LOCAL "-%u process", protocolId),
                 (TimeMSec)(cfgOptionDbl(cfgOptProtocolTimeout) * 1000));
             execOpen(protocolHelperClient->exec);
@@ -240,32 +271,68 @@ protocolRemoteParam(ProtocolStorageType protocolStorageType, unsigned int protoc
         optionReplace, VARSTR(CFGOPT_CONFIG_PATH_STR),
         cfgOptionSource(optConfigPath) != cfgSourceDefault ? cfgOption(optConfigPath) : NULL);
 
-    // Use a C remote
-    kvPut(optionReplace, VARSTR(CFGOPT_C_STR), VARBOOL(true));
+    // Update/remove repo/pg options that are sent to the remote
+    const String *repoHostPrefix = STR(cfgDefOptionName(cfgDefOptRepoHost));
+    const String *repoPrefix = strNewFmt("%s-", PROTOCOL_REMOTE_TYPE_REPO);
+    const String *pgHostPrefix = STR(cfgDefOptionName(cfgDefOptPgHost));
+    const String *pgPrefix = strNewFmt("%s-", PROTOCOL_REMOTE_TYPE_PG);
 
-    // Copy pg options to index 0 since that's what the remote will be expecting
-    if (hostIdx != 0)
+    for (ConfigOption optionId = 0; optionId < CFG_OPTION_TOTAL; optionId++)
     {
-        kvPut(optionReplace, VARSTR(CFGOPT_PG1_PATH_STR), cfgOption(cfgOptPgPath + hostIdx));
+        const String *optionDefName = STR(cfgDefOptionName(cfgOptionDefIdFromId(optionId)));
+        bool remove = false;
 
-        if (cfgOptionSource(cfgOptPgSocketPath + hostIdx) != cfgSourceDefault)
-            kvPut(optionReplace, VARSTR(CFGOPT_PG1_SOCKET_PATH_STR), cfgOption(cfgOptPgSocketPath + hostIdx));
+        // Remove repo host options that are not needed on the remote.  The remote is not expecting to see host settings and it
+        // could get confused about the locality of the repo, i.e. local or remote.
+        if (strBeginsWith(optionDefName, repoHostPrefix))
+        {
+            remove = true;
+        }
+        // Remove repo options when the remote type is pg since they won't be used
+        else if (strBeginsWith(optionDefName, repoPrefix))
+        {
+            if (protocolStorageType == protocolStorageTypePg)
+                remove = true;
+        }
+        // Remove pg host options that are not needed on the remote.  The remote is not expecting to see host settings and it could
+        // get confused about the locality of pg, i.e. local or remote.
+        else if (strBeginsWith(optionDefName, pgHostPrefix))
+        {
+            remove = true;
+        }
+        else if (strBeginsWith(optionDefName, pgPrefix))
+        {
+            // Remove pg options when the remote type is repo since they won't be used
+            if (protocolStorageType == protocolStorageTypeRepo)
+            {
+                remove = true;
+            }
+            // Else move/remove pg options with index > 0 since they won't be used
+            else if (cfgOptionIndex(optionId) > 0)
+            {
+                // If the option index matches the host-id then this is a pg option that the remote needs.  Since the remote expects
+                // to find pg options in index 0, copy the option to index 0.
+                if (cfgOptionIndex(optionId) == hostIdx)
+                {
+                    kvPut(
+                        optionReplace, VARSTRZ(cfgOptionName(optionId - hostIdx)),
+                        cfgOptionSource(optionId) != cfgSourceDefault ? cfgOption(optionId) : NULL);
+                }
 
-        if (cfgOptionSource(cfgOptPgPort + hostIdx) != cfgSourceDefault)
-            kvPut(optionReplace, VARSTR(CFGOPT_PG1_PORT_STR), cfgOption(cfgOptPgPort + hostIdx));
+                // Remove pg options that are not needed on the remote.  The remote is only going to look at index 0 so the options
+                // in higher indexes will not be used and just add clutter which makes debugging harder.
+                remove = true;
+            }
+        }
+
+        // Remove options that have been marked for removal if they are not already null or invalid. This is more efficient because
+        // cfgExecParam() won't have to search through as large a list looking for overrides.
+        if (remove && cfgOptionTest(optionId))
+            kvPut(optionReplace, VARSTRZ(cfgOptionName(optionId)), NULL);
     }
 
-    // Remove pg options that are not needed on the remote.  This is to reduce clustter and make debugging options easier.
-    for (unsigned int pgIdx = 1; pgIdx < cfgOptionIndexTotal(cfgOptPgPath); pgIdx++)
-    {
-        kvPut(optionReplace, VARSTRZ(cfgOptionName(cfgOptPgPath + pgIdx)), NULL);
-        kvPut(optionReplace, VARSTRZ(cfgOptionName(cfgOptPgSocketPath + pgIdx)), NULL);
-        kvPut(optionReplace, VARSTRZ(cfgOptionName(cfgOptPgPort + pgIdx)), NULL);
-    }
-
-    // Add the command option (or use the current command option if it is valid)
-    if (!cfgOptionTest(cfgOptCommand))
-        kvPut(optionReplace, VARSTR(CFGOPT_COMMAND_STR), VARSTRZ(cfgCommandName(cfgCommand())));
+    // Don't pass host-id to the remote.  The host will always be in index 0.
+    kvPut(optionReplace, VARSTR(CFGOPT_HOST_ID_STR), NULL);
 
     // Add the process id (or use the current process id if it is valid)
     if (!cfgOptionTest(cfgOptProcess))
@@ -275,6 +342,17 @@ protocolRemoteParam(ProtocolStorageType protocolStorageType, unsigned int protoc
     kvPut(optionReplace, VARSTR(CFGOPT_LOG_PATH_STR), NULL);
     kvPut(optionReplace, VARSTR(CFGOPT_LOCK_PATH_STR), NULL);
 
+    // ??? Don't pass restore options which the remote doesn't need and are likely to contain spaces because they might get mangled
+    // on the way to the remote depending on how SSH is set up on the server.  This code should be removed when option passing with
+    // spaces is resolved.
+    kvPut(optionReplace, VARSTR(CFGOPT_TYPE_STR), NULL);
+    kvPut(optionReplace, VARSTR(CFGOPT_TARGET_STR), NULL);
+    kvPut(optionReplace, VARSTR(CFGOPT_TARGET_EXCLUSIVE_STR), NULL);
+    kvPut(optionReplace, VARSTR(CFGOPT_TARGET_ACTION_STR), NULL);
+    kvPut(optionReplace, VARSTR(CFGOPT_TARGET_STR), NULL);
+    kvPut(optionReplace, VARSTR(CFGOPT_TARGET_TIMELINE_STR), NULL);
+    kvPut(optionReplace, VARSTR(CFGOPT_RECOVERY_OPTION_STR), NULL);
+
     // Only enable file logging on the remote when requested
     kvPut(
         optionReplace, VARSTR(CFGOPT_LOG_LEVEL_FILE_STR),
@@ -283,10 +361,13 @@ protocolRemoteParam(ProtocolStorageType protocolStorageType, unsigned int protoc
     // Always output errors on stderr for debugging purposes
     kvPut(optionReplace, VARSTR(CFGOPT_LOG_LEVEL_STDERR_STR), VARSTRDEF("error"));
 
-    // Add the type
-    kvPut(optionReplace, VARSTR(CFGOPT_TYPE_STR), isRepo ? VARSTRDEF("backup") : VARSTRDEF("db"));
+    // Disable output to stdout since it is used by the protocol
+    kvPut(optionReplace, VARSTR(CFGOPT_LOG_LEVEL_CONSOLE_STR), VARSTRDEF("off"));
 
-    StringList *commandExec = cfgExecParam(cfgCmdRemote, optionReplace);
+    // Add the remote type
+    kvPut(optionReplace, VARSTR(CFGOPT_REMOTE_TYPE_STR), VARSTR(protocolStorageTypeStr(protocolStorageType)));
+
+    StringList *commandExec = cfgExecParam(cfgCommand(), cfgCmdRoleRemote, optionReplace, false, true);
     strLstInsert(commandExec, 0, cfgOptionStr(isRepo ? cfgOptRepoHostCmd : cfgOptPgHostCmd + hostIdx));
     strLstAdd(result, strLstJoin(commandExec, " "));
 
@@ -304,6 +385,8 @@ protocolRemoteGet(ProtocolStorageType protocolStorageType, unsigned int hostId)
         FUNCTION_LOG_PARAM(UINT, hostId);
     FUNCTION_LOG_END();
 
+    ASSERT(hostId > 0);
+
     // Is this a repo remote?
     bool isRepo = protocolStorageType == protocolStorageTypeRepo;
 
@@ -320,8 +403,10 @@ protocolRemoteGet(ProtocolStorageType protocolStorageType, unsigned int hostId)
             ASSERT(cfgDefOptionIndexTotal(cfgDefOptPgPath) >= cfgDefOptionIndexTotal(cfgDefOptRepoPath));
 
             protocolHelper.clientRemoteSize = cfgDefOptionIndexTotal(cfgDefOptPgPath) + 1;
-            protocolHelper.clientRemote = (ProtocolHelperClient *)memNew(
-                protocolHelper.clientRemoteSize * sizeof(ProtocolHelperClient));
+            protocolHelper.clientRemote = memNew(protocolHelper.clientRemoteSize * sizeof(ProtocolHelperClient));
+
+            for (unsigned int clientIdx = 0; clientIdx < protocolHelper.clientRemoteSize; clientIdx++)
+                protocolHelper.clientRemote[clientIdx] = (ProtocolHelperClient){.exec = NULL};
         }
         MEM_CONTEXT_END();
     }
@@ -385,6 +470,33 @@ protocolRemoteGet(ProtocolStorageType protocolStorageType, unsigned int hostId)
     FUNCTION_LOG_RETURN(PROTOCOL_CLIENT, protocolHelperClient->client);
 }
 
+/**********************************************************************************************************************************/
+void
+protocolRemoteFree(unsigned int hostId)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(UINT, hostId);
+    FUNCTION_LOG_END();
+
+    ASSERT(hostId > 0);
+
+    if (protocolHelper.clientRemote != NULL)
+    {
+        ProtocolHelperClient *protocolHelperClient = &protocolHelper.clientRemote[hostId - 1];
+
+        if (protocolHelperClient->client != NULL)
+        {
+            protocolClientFree(protocolHelperClient->client);
+            execFree(protocolHelperClient->exec);
+
+            protocolHelperClient->client = NULL;
+            protocolHelperClient->exec = NULL;
+        }
+    }
+
+    FUNCTION_LOG_RETURN_VOID();
+}
+
 /***********************************************************************************************************************************
 Send keepalives to all remotes
 ***********************************************************************************************************************************/
@@ -405,6 +517,45 @@ protocolKeepAlive(void)
     FUNCTION_LOG_RETURN_VOID();
 }
 
+/***********************************************************************************************************************************
+Getters
+***********************************************************************************************************************************/
+ProtocolStorageType
+protocolStorageTypeEnum(const String *type)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STRING, type);
+    FUNCTION_TEST_END();
+
+    ASSERT(type != NULL);
+
+    if (strEq(type, PROTOCOL_REMOTE_TYPE_PG_STR))
+        FUNCTION_TEST_RETURN(protocolStorageTypePg);
+    else if (strEq(type, PROTOCOL_REMOTE_TYPE_REPO_STR))
+        FUNCTION_TEST_RETURN(protocolStorageTypeRepo);
+
+    THROW_FMT(AssertError, "invalid protocol storage type '%s'", strPtr(type));
+}
+
+const String *
+protocolStorageTypeStr(ProtocolStorageType type)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(ENUM, type);
+    FUNCTION_TEST_END();
+
+    switch (type)
+    {
+        case protocolStorageTypePg:
+            FUNCTION_TEST_RETURN(PROTOCOL_REMOTE_TYPE_PG_STR);
+
+        case protocolStorageTypeRepo:
+            FUNCTION_TEST_RETURN(PROTOCOL_REMOTE_TYPE_REPO_STR);
+    }
+
+    THROW_FMT(AssertError, "invalid protocol storage type %u", type);
+}
+
 /***********************************************************************************************************************************
 Free the protocol objects and shutdown processes
 ***********************************************************************************************************************************/
@@ -416,19 +567,8 @@ protocolFree(void)
     if (protocolHelper.memContext != NULL)
     {
         // Free remotes
-        for (unsigned int clientIdx  = 0; clientIdx < protocolHelper.clientRemoteSize; clientIdx++)
-        {
-            ProtocolHelperClient *protocolHelperClient = &protocolHelper.clientRemote[clientIdx];
-
-            if (protocolHelperClient->client != NULL)
-            {
-                protocolClientFree(protocolHelperClient->client);
-                execFree(protocolHelperClient->exec);
-
-                protocolHelperClient->client = NULL;
-                protocolHelperClient->exec = NULL;
-            }
-        }
+        for (unsigned int clientIdx = 0; clientIdx < protocolHelper.clientRemoteSize; clientIdx++)
+            protocolRemoteFree(clientIdx + 1);
 
         // Free locals
         for (unsigned int clientIdx  = 0; clientIdx < protocolHelper.clientLocalSize; clientIdx++)
@@ -440,8 +580,7 @@ protocolFree(void)
                 protocolClientFree(protocolHelperClient->client);
                 execFree(protocolHelperClient->exec);
 
-                protocolHelperClient->client = NULL;
-                protocolHelperClient->exec = NULL;
+                *protocolHelperClient = (ProtocolHelperClient){.exec = NULL};
             }
         }
     }
