@@ -23,18 +23,29 @@ Constants
 #define PROTOCOL_SERVICE_REMOTE                                     "remote"
     STRING_DECLARE(PROTOCOL_SERVICE_REMOTE_STR);
 
+#define PROTOCOL_REMOTE_TYPE_PG                                     "pg"
+#define PROTOCOL_REMOTE_TYPE_REPO                                   "repo"
+
 /***********************************************************************************************************************************
 Functions
 ***********************************************************************************************************************************/
 void protocolKeepAlive(void);
-ProtocolClient *protocolLocalGet(ProtocolStorageType protocolStorageType, unsigned int protocolId);
+ProtocolClient *protocolLocalGet(ProtocolStorageType protocolStorageType, unsigned int hostId, unsigned int protocolId);
 ProtocolClient *protocolRemoteGet(ProtocolStorageType protocolStorageType, unsigned int hostId);
 
+// Free (shutdown) a remote
+void protocolRemoteFree(unsigned int hostId);
+
 /***********************************************************************************************************************************
 Getters
 ***********************************************************************************************************************************/
 bool pgIsLocal(unsigned int hostId);
 bool repoIsLocal(void);
+void repoIsLocalVerify(void);
+
+// Get enum/string for protocol storage type
+ProtocolStorageType protocolStorageTypeEnum(const String *type);
+const String *protocolStorageTypeStr(ProtocolStorageType type);
 
 /***********************************************************************************************************************************
 Destructor
