@@ -4,7 +4,6 @@ S3 Storage
 #include "build.auto.h"
 
 #include <string.h>
-#include <time.h>
 
 #include "common/crypto/hash.h"
 #include "common/encode.h"
@@ -59,6 +58,7 @@ STRING_STATIC(S3_XML_TAG_CONTENTS_STR,                              "Contents");
 STRING_STATIC(S3_XML_TAG_DELETE_STR,                                "Delete");
 STRING_STATIC(S3_XML_TAG_ERROR_STR,                                 "Error");
 STRING_STATIC(S3_XML_TAG_KEY_STR,                                   "Key");
+STRING_STATIC(S3_XML_TAG_LAST_MODIFIED_STR,                         "LastModified");
 STRING_STATIC(S3_XML_TAG_MESSAGE_STR,                               "Message");
 STRING_STATIC(S3_XML_TAG_NEXT_CONTINUATION_TOKEN_STR,               "NextContinuationToken");
 STRING_STATIC(S3_XML_TAG_OBJECT_STR,                                "Object");
@@ -86,9 +86,10 @@ Object type
 ***********************************************************************************************************************************/
 struct StorageS3
 {
+    STORAGE_COMMON_MEMBER;
     MemContext *memContext;
     HttpClientCache *httpClientCache;                               // Http client cache to service requests
-    const StringList *headerRedactList;                             // List of headers to redact from logging
+    StringList *headerRedactList;                                   // List of headers to redact from logging
 
     const String *bucket;                                           // Bucket to store data in
     const String *region;                                           // e.g. us-east-1
@@ -97,6 +98,7 @@ struct StorageS3
     const String *securityToken;                                    // Security token, if any
     size_t partSize;                                                // Part size for multi-part upload
     unsigned int deleteMax;                                         // Maximum objects that can be deleted in one request
+    StorageS3UriStyle uriStyle;                                     // Path or host style URIs
     const String *bucketEndpoint;                                   // Set to {bucket}.{endpoint}
     unsigned int port;                                              // Host port
 
@@ -117,7 +119,7 @@ static String *
 storageS3DateTime(time_t authTime)
 {
     FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(INT64, authTime);
+        FUNCTION_TEST_PARAM(TIME, authTime);
     FUNCTION_TEST_END();
 
     char buffer[ISO_8601_DATE_TIME_SIZE + 1];
@@ -257,6 +259,10 @@ storageS3Request(
     unsigned int retryRemaining = 2;
     bool done;
 
+    // When using path-style URIs the bucket name needs to be prepended
+    if (this->uriStyle == storageS3UriStylePath)
+        uri = strNewFmt("/%s%s", strPtr(this->bucket), strPtr(uri));
+
     do
     {
         done = true;
@@ -282,9 +288,7 @@ storageS3Request(
             // Generate authorization header
             storageS3Auth(
                 this, verb, httpUriEncode(uri, true), query, storageS3DateTime(time(NULL)), requestHeader,
-                body == NULL || bufUsed(body) == 0 ?
-                    STRDEF("e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855") :
-                    bufHex(cryptoHashOne(HASH_TYPE_SHA256_STR, body)));
+                body == NULL || bufUsed(body) == 0 ? HASH_TYPE_SHA256_ZERO_STR : bufHex(cryptoHashOne(HASH_TYPE_SHA256_STR, body)));
 
             // Get an http client
             HttpClient *httpClient = httpClientCacheGet(this->httpClientCache);
@@ -307,7 +311,7 @@ storageS3Request(
 
                         if (strEq(errorCode, S3_ERROR_REQUEST_TIME_TOO_SKEWED_STR))
                         {
-                            LOG_DEBUG(
+                            LOG_DEBUG_FMT(
                                 "retry %s: %s", strPtr(errorCode),
                                 strPtr(xmlNodeContent(xmlNodeChild(error, S3_XML_TAG_MESSAGE_STR, true))));
 
@@ -354,14 +358,15 @@ storageS3Request(
                     }
 
                     // Output response headers
-                    const HttpHeader *responseHeader = httpClientReponseHeader(httpClient);
+                    const HttpHeader *responseHeader = httpClientResponseHeader(httpClient);
                     const StringList *responseHeaderList = httpHeaderList(responseHeader);
 
                     if (strLstSize(responseHeaderList) > 0)
                     {
                         strCat(error, "\n*** Response Headers ***:");
 
-                        for (unsigned int responseHeaderIdx = 0; responseHeaderIdx < strLstSize(responseHeaderList); responseHeaderIdx++)
+                        for (unsigned int responseHeaderIdx = 0; responseHeaderIdx < strLstSize(responseHeaderList);
+                                responseHeaderIdx++)
                         {
                             const String *key = strLstGet(responseHeaderList, responseHeaderIdx);
                             strCatFmt(error, "\n%s: %s", strPtr(key), strPtr(httpHeaderGet(responseHeader, key)));
@@ -377,10 +382,11 @@ storageS3Request(
             }
             else
             {
-                // On success move the buffer to the calling context
+                // On success move the buffer to the prior context
                 result.httpClient = httpClient;
-                result.responseHeader = httpHeaderMove(httpHeaderDup(httpClientReponseHeader(httpClient), NULL), MEM_CONTEXT_OLD());
-                result.response = bufMove(response, MEM_CONTEXT_OLD());
+                result.responseHeader = httpHeaderMove(
+                    httpHeaderDup(httpClientResponseHeader(httpClient), NULL), memContextPrior());
+                result.response = bufMove(response, memContextPrior());
             }
 
         }
@@ -508,9 +514,11 @@ storageS3ListInternal(
                 }
 
                 // Get the continuation token and store it in the outer temp context
-                memContextSwitch(MEM_CONTEXT_OLD());
-                continuationToken = xmlNodeContent(xmlNodeChild(xmlRoot, S3_XML_TAG_NEXT_CONTINUATION_TOKEN_STR, false));
-                memContextSwitch(MEM_CONTEXT_TEMP());
+                MEM_CONTEXT_PRIOR_BEGIN()
+                {
+                    continuationToken = xmlNodeContent(xmlNodeChild(xmlRoot, S3_XML_TAG_NEXT_CONTINUATION_TOKEN_STR, false));
+                }
+                MEM_CONTEXT_PRIOR_END();
             }
             MEM_CONTEXT_TEMP_END();
         }
@@ -521,17 +529,16 @@ storageS3ListInternal(
     FUNCTION_LOG_RETURN_VOID();
 }
 
-/***********************************************************************************************************************************
-Does a file exist? This function is only for files, not paths.
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 static bool
-storageS3Exists(THIS_VOID, const String *file)
+storageS3Exists(THIS_VOID, const String *file, StorageInterfaceExistsParam param)
 {
     THIS(StorageS3);
 
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE_S3, this);
         FUNCTION_LOG_PARAM(STRING, file);
+        (void)param;                                                // No parameters are used
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
@@ -548,22 +555,20 @@ storageS3Exists(THIS_VOID, const String *file)
     FUNCTION_LOG_RETURN(BOOL, result);
 }
 
-/***********************************************************************************************************************************
-File info
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 static StorageInfo
-storageS3Info(THIS_VOID, const String *file, bool followLink)
+storageS3Info(THIS_VOID, const String *file, StorageInterfaceInfoParam param)
 {
     THIS(StorageS3);
 
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(STORAGE_S3, this);
         FUNCTION_LOG_PARAM(STRING, file);
+        (void)param;                                                // No parameters are used
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
     ASSERT(file != NULL);
-    (void)followLink;
 
     StorageInfo result = {0};
 
@@ -576,20 +581,35 @@ storageS3Info(THIS_VOID, const String *file, bool followLink)
         result.exists = true;
         result.type = storageTypeFile;
         result.size = cvtZToUInt64(strPtr(httpHeaderGet(httpResult.responseHeader, HTTP_HEADER_CONTENT_LENGTH_STR)));
+        result.timeModified = httpLastModifiedToTime(httpHeaderGet(httpResult.responseHeader, HTTP_HEADER_LAST_MODIFIED_STR));
     }
 
     FUNCTION_LOG_RETURN(STORAGE_INFO, result);
 }
 
-/***********************************************************************************************************************************
-Info for all files/paths in a path
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 typedef struct StorageS3InfoListData
 {
     StorageInfoListCallback callback;                               // User-supplied callback function
     void *callbackData;                                             // User-supplied callback data
 } StorageS3InfoListData;
 
+// Helper to convert YYYY-MM-DDTHH:MM:SS.MSECZ format to time_t.  This format is very nearly ISO-8601 except for the inclusion of
+// milliseconds which are discarded here.
+static time_t
+storageS3CvtTime(const String *time)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STRING, time);
+    FUNCTION_TEST_END();
+
+    FUNCTION_TEST_RETURN(
+        epochFromParts(
+            cvtZToInt(strPtr(strSubN(time, 0, 4))), cvtZToInt(strPtr(strSubN(time, 5, 2))),
+            cvtZToInt(strPtr(strSubN(time, 8, 2))), cvtZToInt(strPtr(strSubN(time, 11, 2))),
+            cvtZToInt(strPtr(strSubN(time, 14, 2))), cvtZToInt(strPtr(strSubN(time, 17, 2)))));
+}
+
 static void
 storageS3InfoListCallback(StorageS3 *this, void *callbackData, const String *name, StorageType type, const XmlNode *xml)
 {
@@ -613,6 +633,8 @@ storageS3InfoListCallback(StorageS3 *this, void *callbackData, const String *nam
         .type = type,
         .name = name,
         .size = type == storageTypeFile ? cvtZToUInt64(strPtr(xmlNodeContent(xmlNodeChild(xml, S3_XML_TAG_SIZE_STR, true)))) : 0,
+        .timeModified = type == storageTypeFile ?
+            storageS3CvtTime(xmlNodeContent(xmlNodeChild(xml, S3_XML_TAG_LAST_MODIFIED_STR, true))) : 0,
     };
 
     data->callback(data->callbackData, &info);
@@ -621,7 +643,8 @@ storageS3InfoListCallback(StorageS3 *this, void *callbackData, const String *nam
 }
 
 static bool
-storageS3InfoList(THIS_VOID, const String *path, StorageInfoListCallback callback, void *callbackData)
+storageS3InfoList(
+    THIS_VOID, const String *path, StorageInfoListCallback callback, void *callbackData, StorageInterfaceInfoListParam param)
 {
     THIS(StorageS3);
 
@@ -630,6 +653,7 @@ storageS3InfoList(THIS_VOID, const String *path, StorageInfoListCallback callbac
         FUNCTION_LOG_PARAM(STRING, path);
         FUNCTION_LOG_PARAM(FUNCTIONP, callback);
         FUNCTION_LOG_PARAM_P(VOID, callbackData);
+        (void)param;                                                // No parameters are used
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
@@ -646,9 +670,7 @@ storageS3InfoList(THIS_VOID, const String *path, StorageInfoListCallback callbac
     FUNCTION_LOG_RETURN(BOOL, true);
 }
 
-/***********************************************************************************************************************************
-Get a list of files from a directory
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 static void
 storageS3ListCallback(StorageS3 *this, void *callbackData, const String *name, StorageType type, const XmlNode *xml)
 {
@@ -672,14 +694,14 @@ storageS3ListCallback(StorageS3 *this, void *callbackData, const String *name, S
 }
 
 static StringList *
-storageS3List(THIS_VOID, const String *path, const String *expression)
+storageS3List(THIS_VOID, const String *path, StorageInterfaceListParam param)
 {
     THIS(StorageS3);
 
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE_S3, this);
         FUNCTION_LOG_PARAM(STRING, path);
-        FUNCTION_LOG_PARAM(STRING, expression);
+        FUNCTION_LOG_PARAM(STRING, param.expression);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
@@ -691,19 +713,17 @@ storageS3List(THIS_VOID, const String *path, const String *expression)
     {
         result = strLstNew();
 
-        storageS3ListInternal(this, path, expression, false, storageS3ListCallback, result);
-        strLstMove(result, MEM_CONTEXT_OLD());
+        storageS3ListInternal(this, path, param.expression, false, storageS3ListCallback, result);
+        strLstMove(result, memContextPrior());
     }
     MEM_CONTEXT_TEMP_END();
 
     FUNCTION_LOG_RETURN(STRING_LIST, result);
 }
 
-/***********************************************************************************************************************************
-New file read object
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 static StorageRead *
-storageS3NewRead(THIS_VOID, const String *file, bool ignoreMissing, bool compressible)
+storageS3NewRead(THIS_VOID, const String *file, bool ignoreMissing, StorageInterfaceNewReadParam param)
 {
     THIS(StorageS3);
 
@@ -711,7 +731,7 @@ storageS3NewRead(THIS_VOID, const String *file, bool ignoreMissing, bool compres
         FUNCTION_LOG_PARAM(STORAGE_S3, this);
         FUNCTION_LOG_PARAM(STRING, file);
         FUNCTION_LOG_PARAM(BOOL, ignoreMissing);
-        (void)compressible;
+        (void)param;                                                // No parameters are used
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
@@ -720,44 +740,29 @@ storageS3NewRead(THIS_VOID, const String *file, bool ignoreMissing, bool compres
     FUNCTION_LOG_RETURN(STORAGE_READ, storageReadS3New(this, file, ignoreMissing));
 }
 
-/***********************************************************************************************************************************
-New file write object
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 static StorageWrite *
-storageS3NewWrite(
-    THIS_VOID, const String *file, mode_t modeFile, mode_t modePath, const String *user, const String *group, time_t timeModified,
-    bool createPath, bool syncFile, bool syncPath, bool atomic, bool compressible)
+storageS3NewWrite(THIS_VOID, const String *file, StorageInterfaceNewWriteParam param)
 {
     THIS(StorageS3);
 
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE_S3, this);
         FUNCTION_LOG_PARAM(STRING, file);
-        FUNCTION_LOG_PARAM(MODE, modeFile);
-        FUNCTION_LOG_PARAM(MODE, modePath);
-        FUNCTION_LOG_PARAM(STRING, user);
-        FUNCTION_LOG_PARAM(STRING, group);
-        FUNCTION_LOG_PARAM(INT64, timeModified);
-        FUNCTION_LOG_PARAM(BOOL, createPath);
-        FUNCTION_LOG_PARAM(BOOL, syncFile);
-        FUNCTION_LOG_PARAM(BOOL, syncPath);
-        FUNCTION_LOG_PARAM(BOOL, atomic);
-        (void)compressible;
+        (void)param;                                                // No parameters are used
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
     ASSERT(file != NULL);
-    ASSERT(createPath);
-    ASSERT(user == NULL);
-    ASSERT(group == NULL);
-    ASSERT(timeModified == 0);
+    ASSERT(param.createPath);
+    ASSERT(param.user == NULL);
+    ASSERT(param.group == NULL);
+    ASSERT(param.timeModified == 0);
 
     FUNCTION_LOG_RETURN(STORAGE_WRITE, storageWriteS3New(this, file, this->partSize));
 }
 
-/***********************************************************************************************************************************
-Remove a path
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 typedef struct StorageS3PathRemoveData
 {
     MemContext *memContext;                                         // Mem context to create xml document in
@@ -824,12 +829,12 @@ storageS3PathRemoveCallback(StorageS3 *this, void *callbackData, const String *n
         // If there is something to delete then create the request
         if (data->xml == NULL)
         {
-            MemContext *memContextOld = memContextSwitch(data->memContext);
-
-            data->xml = xmlDocumentNew(S3_XML_TAG_DELETE_STR);
-            xmlNodeContentSet(xmlNodeAdd(xmlDocumentRoot(data->xml), S3_XML_TAG_QUIET_STR), TRUE_STR);
-
-            memContextSwitch(memContextOld);
+            MEM_CONTEXT_BEGIN(data->memContext)
+            {
+                data->xml = xmlDocumentNew(S3_XML_TAG_DELETE_STR);
+                xmlNodeContentSet(xmlNodeAdd(xmlDocumentRoot(data->xml), S3_XML_TAG_QUIET_STR), TRUE_STR);
+            }
+            MEM_CONTEXT_END();
         }
 
         // Add to delete list
@@ -853,7 +858,7 @@ storageS3PathRemoveCallback(StorageS3 *this, void *callbackData, const String *n
 }
 
 static bool
-storageS3PathRemove(THIS_VOID, const String *path, bool recurse)
+storageS3PathRemove(THIS_VOID, const String *path, bool recurse, StorageInterfacePathRemoveParam param)
 {
     THIS(StorageS3);
 
@@ -861,6 +866,7 @@ storageS3PathRemove(THIS_VOID, const String *path, bool recurse)
         FUNCTION_LOG_PARAM(STORAGE_S3, this);
         FUNCTION_LOG_PARAM(STRING, path);
         FUNCTION_LOG_PARAM(BOOL, recurse);
+        (void)param;                                                // No parameters are used
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
@@ -879,23 +885,21 @@ storageS3PathRemove(THIS_VOID, const String *path, bool recurse)
     FUNCTION_LOG_RETURN(BOOL, true);
 }
 
-/***********************************************************************************************************************************
-Remove a file
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 static void
-storageS3Remove(THIS_VOID, const String *file, bool errorOnMissing)
+storageS3Remove(THIS_VOID, const String *file, StorageInterfaceRemoveParam param)
 {
     THIS(StorageS3);
 
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE_S3, this);
         FUNCTION_LOG_PARAM(STRING, file);
-        FUNCTION_LOG_PARAM(BOOL, errorOnMissing);
+        FUNCTION_LOG_PARAM(BOOL, param.errorOnMissing);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
     ASSERT(file != NULL);
-    ASSERT(!errorOnMissing);
+    ASSERT(!param.errorOnMissing);
 
     storageS3Request(this, HTTP_VERB_DELETE_STR, file, NULL, NULL, true, false);
 
@@ -905,12 +909,24 @@ storageS3Remove(THIS_VOID, const String *file, bool errorOnMissing)
 /***********************************************************************************************************************************
 New object
 ***********************************************************************************************************************************/
+static const StorageInterface storageInterfaceS3 =
+{
+    .exists = storageS3Exists,
+    .info = storageS3Info,
+    .infoList = storageS3InfoList,
+    .list = storageS3List,
+    .newRead = storageS3NewRead,
+    .newWrite = storageS3NewWrite,
+    .pathRemove = storageS3PathRemove,
+    .remove = storageS3Remove,
+};
+
 Storage *
 storageS3New(
     const String *path, bool write, StoragePathExpressionCallback pathExpressionFunction, const String *bucket,
-    const String *endPoint, const String *region, const String *accessKey, const String *secretAccessKey,
-    const String *securityToken, size_t partSize, unsigned int deleteMax, const String *host, unsigned int port, TimeMSec timeout,
-    bool verifyPeer, const String *caFile, const String *caPath)
+    const String *endPoint, StorageS3UriStyle uriStyle, const String *region, const String *accessKey,
+    const String *secretAccessKey, const String *securityToken, size_t partSize, unsigned int deleteMax, const String *host,
+    unsigned int port, TimeMSec timeout, bool verifyPeer, const String *caFile, const String *caPath)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STRING, path);
@@ -918,6 +934,7 @@ storageS3New(
         FUNCTION_LOG_PARAM(FUNCTIONP, pathExpressionFunction);
         FUNCTION_LOG_PARAM(STRING, bucket);
         FUNCTION_LOG_PARAM(STRING, endPoint);
+        FUNCTION_LOG_PARAM(ENUM, uriStyle);
         FUNCTION_LOG_PARAM(STRING, region);
         FUNCTION_TEST_PARAM(STRING, accessKey);
         FUNCTION_TEST_PARAM(STRING, secretAccessKey);
@@ -943,31 +960,37 @@ storageS3New(
     MEM_CONTEXT_NEW_BEGIN("StorageS3")
     {
         StorageS3 *driver = memNew(sizeof(StorageS3));
-        driver->memContext = MEM_CONTEXT_NEW();
 
-        driver->bucket = strDup(bucket);
-        driver->region = strDup(region);
-        driver->accessKey = strDup(accessKey);
-        driver->secretAccessKey = strDup(secretAccessKey);
-        driver->securityToken = strDup(securityToken);
-        driver->partSize = partSize;
-        driver->deleteMax = deleteMax;
-        driver->bucketEndpoint = strNewFmt("%s.%s", strPtr(bucket), strPtr(endPoint));
-        driver->port = port;
+        *driver = (StorageS3)
+        {
+            .memContext = MEM_CONTEXT_NEW(),
+            .interface = storageInterfaceS3,
+            .bucket = strDup(bucket),
+            .region = strDup(region),
+            .accessKey = strDup(accessKey),
+            .secretAccessKey = strDup(secretAccessKey),
+            .securityToken = strDup(securityToken),
+            .partSize = partSize,
+            .deleteMax = deleteMax,
+            .uriStyle = uriStyle,
+            .bucketEndpoint = uriStyle == storageS3UriStyleHost ?
+                strNewFmt("%s.%s", strPtr(bucket), strPtr(endPoint)) : strDup(endPoint),
+            .port = port,
 
-        // Force the signing key to be generated on the first run
-        driver->signingKeyDate = YYYYMMDD_STR;
+            // Force the signing key to be generated on the first run
+            .signingKeyDate = YYYYMMDD_STR,
+        };
 
         // Create the http client cache used to service requests
         driver->httpClientCache = httpClientCacheNew(
             host == NULL ? driver->bucketEndpoint : host, driver->port, timeout, verifyPeer, caFile, caPath);
-        driver->headerRedactList = strLstAdd(strLstNew(), S3_HEADER_AUTHORIZATION_STR);
 
-        this = storageNewP(
-            STORAGE_S3_TYPE_STR, path, 0, 0, write, pathExpressionFunction, driver,
-            .exists = storageS3Exists, .info = storageS3Info, .infoList = storageS3InfoList, .list = storageS3List,
-            .newRead = storageS3NewRead, .newWrite = storageS3NewWrite, .pathRemove = storageS3PathRemove,
-            .remove = storageS3Remove);
+        // Create list of redacted headers
+        driver->headerRedactList = strLstNew();
+        strLstAdd(driver->headerRedactList, S3_HEADER_AUTHORIZATION_STR);
+
+        this = storageNew(
+            STORAGE_S3_TYPE_STR, path, 0, 0, write, pathExpressionFunction, driver, driver->interface);
     }
     MEM_CONTEXT_NEW_END();
 
