@@ -21,6 +21,11 @@ use pgBackRest::Version;
 use pgBackRestTest::Common::ContainerTest;
 use pgBackRestTest::Common::ExecuteTest;
 
+####################################################################################################################################
+# Expect log cache for supplemental files so they are only added to expect logs when they have changed
+####################################################################################################################################
+my $hyExpectCache = undef;
+
 ####################################################################################################################################
 # new
 ####################################################################################################################################
@@ -148,15 +153,7 @@ sub supplementalAdd
     my $strComment = shift;
     my $strContent = shift;
 
-    my $strHeader = "+ supplemental file: " . $self->regExpReplaceAll($strFileName);
-
-    if (defined($strComment))
-    {
-        $self->{strLog} .= "\n" . $self->regExpReplaceAll($strComment) . "\n" . ('=' x '132') . "\n";
-    }
-
-    $self->{strLog} .= "\n${strHeader}\n" . ('-' x length($strHeader)) . "\n";
-
+    # Get content if it is not defined
     if (!defined($strContent))
     {
         open(my $hFile, '<', $strFileName)
@@ -164,14 +161,27 @@ sub supplementalAdd
 
         while (my $strLine = readline($hFile))
         {
-            $self->{strLog} .= $self->regExpReplaceAll($strLine);
+            $strContent .= $strLine;
         }
 
         close($hFile);
     }
-    else
+
+    # Only ouput when the content of this file has changed
+    if (!defined($hyExpectCache->{$strFileName}) || $hyExpectCache->{$strFileName} ne $strContent)
     {
-        if (defined($strContent) && length($strContent) > 0)
+        # $hyExpectCache->{$strFileName} = $strContent;
+
+        my $strHeader = "+ supplemental file: " . $self->regExpReplaceAll($strFileName);
+
+        if (defined($strComment))
+        {
+            $self->{strLog} .= "\n" . $self->regExpReplaceAll($strComment) . "\n" . ('=' x '132') . "\n";
+        }
+
+        $self->{strLog} .= "\n${strHeader}\n" . ('-' x length($strHeader)) . "\n";
+
+        if (length($strContent) > 0)
         {
             foreach my $strLine (split("\n", $strContent))
             {
@@ -384,6 +394,7 @@ sub regExpReplaceAll
     $strLine = $self->regExpReplace($strLine, 'HTTP-STATISTICS', 'http statistics\:.*$', '[^\:]+$', false);
 
     $strLine = $self->regExpReplace($strLine, 'GROUP', 'strGroup = [^ \n,\[\]]+', '[^ \n,\[\]]+$');
+    $strLine = $self->regExpReplace($strLine, 'GROUP', 'unknown group in backup manifest mapped to \'[^\']+', '[^\']+$');
     $strLine = $self->regExpReplace($strLine, 'GROUP', 'group"[ ]{0,1}:[ ]{0,1}"[^"]+', '[^"]+$');
     $strLine = $self->regExpReplace($strLine, 'GROUP', 'group=\"[^"]+', '[^"]+$');
     $strLine = $self->regExpReplace($strLine, 'GROUP', 'group for .* was not mapped to a name\, set to .+$', '[^ ]+$');
@@ -391,6 +402,7 @@ sub regExpReplaceAll
     $strLine = $self->regExpReplace($strLine, 'GROUP', TEST_USER . '\, ' . TEST_GROUP, '[^ ]+$');
 
     $strLine = $self->regExpReplace($strLine, 'USER', 'strRemoteHostUser = [^ \n,\[\]]+', '[^ \n,\[\]]+$');
+    $strLine = $self->regExpReplace($strLine, 'USER', 'unknown user in backup manifest mapped to \'[^\']+', '[^\']+$');
     $strLine = $self->regExpReplace($strLine, 'USER', 'strUser = [^ \n,\[\]]+', '[^ \n,\[\]]+$');
     $strLine = $self->regExpReplace($strLine, 'USER', 'user"[ ]{0,1}:[ ]{0,1}"[^"]+', '[^"]+$');
     $strLine = $self->regExpReplace($strLine, 'USER', 'user=\"[^"]+', '[^"]+$');
@@ -427,6 +439,9 @@ sub regExpReplaceAll
     $strLine = $self->regExpReplace($strLine, 'TIMESTAMP', "start\" : [0-9]{10}", '[0-9]{10}$', false);
     $strLine = $self->regExpReplace($strLine, 'TIMESTAMP', "stop\" : [0-9]{10}", '[0-9]{10}$', false);
     $strLine = $self->regExpReplace($strLine, 'TIMESTAMP', TEST_GROUP . '\, [0-9]{10}', '[0-9]{10}$', false);
+    $strLine = $self->regExpReplace(
+        $strLine, 'TIMESTAMP', 'generated by pgBackRest restore on .*$',
+        '[0-9]{4}\-[0-9]{2}\-[0-9]{2} [0-9]{2}\:[0-9]{2}\:[0-9]{2}$', false);
     $strLine = $self->regExpReplace($strLine, 'SIZE', "size\"[ ]{0,1}:[ ]{0,1}[0-9]+", '[0-9]+$', false);
     $strLine = $self->regExpReplace($strLine, 'DELTA', "delta\"[ ]{0,1}:[ ]{0,1}[0-9]+", '[0-9]+$', false);
     $strLine = $self->regExpReplace(
@@ -434,6 +449,12 @@ sub regExpReplaceAll
         "${strTimestampRegExp} / ${strTimestampRegExp}\$", false);
     $strLine = $self->regExpReplace($strLine, 'CHECKSUM', 'checksum=[\"]{0,1}[0-f]{40}', '[0-f]{40}$', false);
 
+    $strLine = $self->regExpReplace($strLine, 'LOG-LEVEL-FILE', 'log-level-file=[a-z]+', '[a-z]+$', false);
+    $strLine = $self->regExpReplace($strLine, 'LOG-SUBPROCESS', 'log-subprocess=[a-z]+', '[a-z]+$', false);
+    $strLine = $self->regExpReplace($strLine, '', ' --log-subprocess', undef, false);
+    $strLine = $self->regExpReplace($strLine, '', ' --no-log-subprocess', undef, false);
+    $strLine = $self->regExpReplace($strLine, 'BUFFER-SIZE', 'buffer-size=[0-9a-z]+', '[0-9a-z]+$', false);
+
     $strLine = $self->regExpReplace($strLine, 'REMOTE-PROCESS-TERMINATED-MESSAGE',
         'remote process terminated.*: (ssh.*|no output from terminated process)$',
         '(ssh.*|no output from terminated process)$', false);
