@@ -78,15 +78,15 @@ sub new
     {
         $strName = HOST_BACKUP;
         $strImage = containerRepo() . ':' . testRunGet()->vm() . '-test';
-        $strUser = testRunGet()->backrestUser();
     }
     else
     {
         $strName = $$oParam{strName};
         $strImage = $$oParam{strImage};
-        $strUser = testRunGet()->pgUser();
     }
 
+    $strUser = testRunGet()->pgUser();
+
     # Create the host
     my $self = $class->SUPER::new($strName, {strImage => $strImage, strUser => $strUser});
     bless $self, $class;
@@ -169,8 +169,6 @@ sub backupBegin
         );
 
     # Set defaults
-    my $strTest = defined($$oParam{strTest}) ? $$oParam{strTest} : undef;
-    my $fTestDelay = defined($$oParam{fTestDelay}) ? $$oParam{fTestDelay} : .2;
     my $oExpectedManifest = defined($$oParam{oExpectedManifest}) ? $$oParam{oExpectedManifest} : undef;
 
     $strComment =
@@ -188,19 +186,12 @@ sub backupBegin
         (defined($$oParam{bStandby}) && $$oParam{bStandby} ? " --backup-standby" : '') .
         (defined($oParam->{strRepoType}) ? " --repo1-type=$oParam->{strRepoType}" : '') .
         ($strType ne 'incr' ? " --type=${strType}" : '') .
-        ' --stanza=' . (defined($oParam->{strStanza}) ? $oParam->{strStanza} : $self->stanza()) . ' backup' .
-        (defined($strTest) ? " --test --test-delay=${fTestDelay} --test-point=" . lc($strTest) . '=y' : ''),
+        ' --stanza=' . (defined($oParam->{strStanza}) ? $oParam->{strStanza} : $self->stanza()) . ' backup',
         {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus},
          oLogTest => $self->{oLogTest}, bLogOutput => $self->synthetic()});
 
     $oExecuteBackup->begin();
 
-    # Return at the test point if one was defined
-    if (defined($strTest))
-    {
-        $oExecuteBackup->end($strTest);
-    }
-
     # Return from function and log return values if any
     return logDebugReturn
     (
@@ -1021,10 +1012,12 @@ sub configCreate
     # General options
     # ------------------------------------------------------------------------------------------------------------------------------
     $oParamHash{&CFGDEF_SECTION_GLOBAL}{cfgOptionName(CFGOPT_LOG_LEVEL_CONSOLE)} = lc(DETAIL);
-    $oParamHash{&CFGDEF_SECTION_GLOBAL}{cfgOptionName(CFGOPT_LOG_LEVEL_FILE)} = lc(TRACE);
+    $oParamHash{&CFGDEF_SECTION_GLOBAL}{cfgOptionName(CFGOPT_LOG_LEVEL_FILE)} = testRunGet()->logLevelTestFile();
     $oParamHash{&CFGDEF_SECTION_GLOBAL}{cfgOptionName(CFGOPT_LOG_LEVEL_STDERR)} = lc(OFF);
-    $oParamHash{&CFGDEF_SECTION_GLOBAL}{cfgOptionName(CFGOPT_LOG_SUBPROCESS)} = 'y';
+    $oParamHash{&CFGDEF_SECTION_GLOBAL}{cfgOptionName(CFGOPT_LOG_SUBPROCESS)} =
+        testRunGet()->logLevelTestFile() eq lc(OFF) ? 'n' : 'y';
     $oParamHash{&CFGDEF_SECTION_GLOBAL}{cfgOptionName(CFGOPT_LOG_TIMESTAMP)} = 'n';
+    $oParamHash{&CFGDEF_SECTION_GLOBAL}{cfgOptionName(CFGOPT_BUFFER_SIZE)} = '64k';
 
     $oParamHash{&CFGDEF_SECTION_GLOBAL}{cfgOptionName(CFGOPT_LOG_PATH)} = $self->logPath();
     $oParamHash{&CFGDEF_SECTION_GLOBAL}{cfgOptionName(CFGOPT_LOCK_PATH)} = $self->lockPath();
@@ -1032,7 +1025,7 @@ sub configCreate
     $oParamHash{&CFGDEF_SECTION_GLOBAL}{cfgOptionName(CFGOPT_PROTOCOL_TIMEOUT)} = 60;
     $oParamHash{&CFGDEF_SECTION_GLOBAL}{cfgOptionName(CFGOPT_DB_TIMEOUT)} = 45;
 
-    # Set to make sure that changinf the default works and to speed compression for testing
+    # Set to make sure that changing the default works and to speed compression for testing
     $oParamHash{&CFGDEF_SECTION_GLOBAL}{cfgOptionName(CFGOPT_COMPRESS_LEVEL)} = 3;
 
     # Only set network compress level if there is more than one host
@@ -1181,9 +1174,6 @@ sub configCreate
 
     # Write out the configuration file
     storageTest()->put($self->backrestConfig(), iniRender(\%oParamHash, true));
-
-    # Modify the file permissions so it can be read/saved by all test users
-    executeTest('sudo chmod 660 ' . $self->backrestConfig());
 }
 
 ####################################################################################################################################
@@ -1217,17 +1207,8 @@ sub configUpdate
         }
     }
 
-    # Modify the file permissions so it can be saved by all test users
-    executeTest(
-        'sudo chmod 660 ' . $self->backrestConfig() . ' && sudo chmod 770 ' . dirname($self->backrestConfig()));
-
     storageTest()->put($self->backrestConfig(), iniRender($oConfig, true));
 
-    # Fix permissions back to original
-    executeTest(
-        'sudo chmod 660 ' . $self->backrestConfig() . ' && sudo chmod 770 ' . dirname($self->backrestConfig()) .
-        ' && sudo chown ' . $self->userGet() . ' ' . $self->backrestConfig());
-
     # Return from function and log return values if any
     return logDebugReturn($strOperation);
 }
@@ -1343,34 +1324,34 @@ sub infoMunge
                 foreach my $strSubKey (keys(%{$hParam->{$strSection}{$strKey}}))
                 {
                     # Munge the copy with the new parameter values
-                    $oMungeIni->set($strSection, $strKey, $strSubKey, $hParam->{$strSection}{$strKey}{$strSubKey});
+                    if (defined($hParam->{$strSection}{$strKey}{$strSubKey}))
+                    {
+                        $oMungeIni->set($strSection, $strKey, $strSubKey, $hParam->{$strSection}{$strKey}{$strSubKey});
+                    }
+                    else
+                    {
+                        $oMungeIni->remove($strSection, $strKey, $strSubKey);
+                    }
                 }
             }
             else
             {
                 # Munge the copy with the new parameter values
-                $oMungeIni->set($strSection, $strKey, undef, $hParam->{$strSection}{$strKey});
+                if (defined($hParam->{$strSection}{$strKey}))
+                {
+                    $oMungeIni->set($strSection, $strKey, undef, $hParam->{$strSection}{$strKey});
+                }
+                else
+                {
+                    $oMungeIni->remove($strSection, $strKey);
+                }
             }
         }
     }
 
-    # Modify the file/directory permissions so it can be saved
-    if ($self->isFS())
-    {
-        executeTest("sudo rm -f ${strFileName}* && sudo chmod 770 " . dirname($strFileName));
-    }
-
     # Save the munged data to the file
     $oMungeIni->save();
 
-    # Fix permissions
-    if ($self->isFS())
-    {
-        executeTest(
-            "sudo chmod 640 ${strFileName}* && sudo chmod 750 " . dirname($strFileName) .
-            ' && sudo chown ' . $self->userGet() . " ${strFileName}*");
-    }
-
     # Clear the cache is requested
     if (!$bCache)
     {
@@ -1410,23 +1391,9 @@ sub infoRestore
     {
         if ($bSave)
         {
-            # Modify the file/directory permissions so it can be saved
-            if ($self->isFS())
-            {
-                executeTest("sudo rm -f ${strFileName}* && sudo chmod 770 " . dirname($strFileName));
-            }
-
             # Save the munged data to the file
             $self->{hInfoFile}{$strFileName}->{bModified} = true;
             $self->{hInfoFile}{$strFileName}->save();
-
-            # Fix permissions
-            if ($self->isFS())
-            {
-                executeTest(
-                    "sudo chmod 640 ${strFileName} && sudo chmod 750 " . dirname($strFileName) .
-                    ' && sudo chown ' . $self->userGet() . " ${strFileName}*");
-            }
         }
     }
     else
@@ -1539,16 +1506,7 @@ sub configRemap
     # Save backup config file (but not if this is the standby which is not the source of backups)
     if (defined($oHostBackup))
     {
-        # Modify the file permissions so it can be read/saved by all test users
-        executeTest(
-            'sudo chmod 660 ' . $oHostBackup->backrestConfig() . ' && sudo chmod 770 ' . dirname($oHostBackup->backrestConfig()));
-
         storageTest()->put($oHostBackup->backrestConfig(), iniRender($oRemoteConfig, true));
-
-        # Fix permissions
-        executeTest(
-            'sudo chmod 660 ' . $oHostBackup->backrestConfig() . ' && sudo chmod 770 ' . dirname($oHostBackup->backrestConfig()) .
-            ' && sudo chown ' . $oHostBackup->userGet() . ' ' . $oHostBackup->backrestConfig());
     }
 }
 
@@ -1923,6 +1881,13 @@ sub restoreCompare
         }
     }
 
+    # If PostgreSQL >= 12 don't compare postgresql.auto.conf since it will have recovery settings written into it
+    if (${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP_DB}{&MANIFEST_KEY_DB_VERSION} >= PG_VERSION_12)
+    {
+        delete($oExpectedManifestRef->{&MANIFEST_SECTION_TARGET_FILE}{'pg_data/postgresql.auto.conf'});
+        $oActualManifest->remove(MANIFEST_SECTION_TARGET_FILE, 'pg_data/postgresql.auto.conf');
+    }
+
     # If the link section is empty then delete it and the default section
     if (keys(%{${$oExpectedManifestRef}{&MANIFEST_SECTION_TARGET_LINK}}) == 0)
     {
