@@ -25,13 +25,11 @@ testRun(void)
     if (testBegin("archiveAsyncStatus()"))
     {
         StringList *argList = strLstNew();
-        strLstAddZ(argList, "pgbackrest");
         strLstAdd(argList, strNewFmt("--spool-path=%s", testPath()));
         strLstAddZ(argList, "--archive-async");
         strLstAddZ(argList, "--archive-timeout=1");
         strLstAddZ(argList, "--stanza=db");
-        strLstAddZ(argList, "archive-push");
-        harnessCfgLoad(strLstSize(argList), strLstPtr(argList));
+        harnessCfgLoad(cfgCmdArchivePush, argList);
 
         // -------------------------------------------------------------------------------------------------------------------------
         String *segment = strNew("000000010000000100000001");
@@ -47,36 +45,36 @@ testRun(void)
         TEST_RESULT_BOOL(archiveAsyncStatus(archiveModePush, segment, false), false, "status file not present");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        storagePutNP(
-            storageNewWriteNP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.ok", strPtr(segment))),
+        storagePutP(
+            storageNewWriteP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.ok", strPtr(segment))),
             BUFSTRDEF(BOGUS_STR));
         TEST_ERROR(
             archiveAsyncStatus(archiveModePush, segment, false), FormatError,
             "000000010000000100000001.ok content must have at least two lines");
 
-        storagePutNP(
-            storageNewWriteNP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.ok", strPtr(segment))),
+        storagePutP(
+            storageNewWriteP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.ok", strPtr(segment))),
             BUFSTRDEF(BOGUS_STR "\n"));
         TEST_ERROR(
             archiveAsyncStatus(archiveModePush, segment, false), FormatError, "000000010000000100000001.ok message must be > 0");
 
-        storagePutNP(
-            storageNewWriteNP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.ok", strPtr(segment))),
+        storagePutP(
+            storageNewWriteP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.ok", strPtr(segment))),
             BUFSTRDEF(BOGUS_STR "\nmessage"));
         TEST_ERROR(
             archiveAsyncStatus(archiveModePush, segment, false), FormatError, "unable to convert base 10 string 'BOGUS' to int");
 
-        storagePutNP(storageNewWriteNP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.ok", strPtr(segment))), NULL);
+        storagePutP(storageNewWriteP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.ok", strPtr(segment))), NULL);
         TEST_RESULT_BOOL(archiveAsyncStatus(archiveModePush, segment, false), true, "ok file");
 
-        storagePutNP(
-            storageNewWriteNP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.ok", strPtr(segment))),
+        storagePutP(
+            storageNewWriteP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.ok", strPtr(segment))),
             BUFSTRDEF("0\nwarning"));
         TEST_RESULT_BOOL(archiveAsyncStatus(archiveModePush, segment, false), true, "ok file with warning");
         harnessLogResult("P00   WARN: warning");
 
-        storagePutNP(
-            storageNewWriteNP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.ok", strPtr(segment))),
+        storagePutP(
+            storageNewWriteP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.ok", strPtr(segment))),
             BUFSTRDEF("25\nerror"));
         TEST_RESULT_BOOL(archiveAsyncStatus(archiveModePush, segment, false), true, "error status renamed to ok");
         harnessLogResult(
@@ -87,22 +85,22 @@ testRun(void)
             "remove ok");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        storagePutNP(
-            storageNewWriteNP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.error", strPtr(segment))), bufNew(0));
+        storagePutP(
+            storageNewWriteP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.error", strPtr(segment))), bufNew(0));
         TEST_ERROR(
             archiveAsyncStatus(archiveModePush, segment, true), AssertError,
             "status file '000000010000000100000001.error' has no content");
 
-        storagePutNP(
-            storageNewWriteNP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.error", strPtr(segment))),
+        storagePutP(
+            storageNewWriteP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s.error", strPtr(segment))),
             BUFSTRDEF("25\nmessage"));
         TEST_ERROR(archiveAsyncStatus(archiveModePush, segment, true), AssertError, "message");
 
         TEST_RESULT_BOOL(archiveAsyncStatus(archiveModePush, segment, false), false, "suppress error");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        storagePutNP(
-            storageNewWriteNP(storageSpoolWrite(), strNew(STORAGE_SPOOL_ARCHIVE_OUT "/global.error")),
+        storagePutP(
+            storageNewWriteP(storageSpoolWrite(), strNew(STORAGE_SPOOL_ARCHIVE_OUT "/global.error")),
             BUFSTRDEF("102\nexecute error"));
 
         TEST_ERROR(archiveAsyncStatus(archiveModePush, strNew("anyfile"), true), ExecuteError, "execute error");
@@ -112,18 +110,17 @@ testRun(void)
     if (testBegin("archiveAsyncStatusErrorWrite() and archiveAsyncStatusOkWrite()"))
     {
         StringList *argList = strLstNew();
-        strLstAddZ(argList, "pgbackrest");
         strLstAdd(argList, strNewFmt("--spool-path=%s", testPath()));
         strLstAddZ(argList, "--stanza=db");
-        strLstAddZ(argList, "archive-get-async");
-        harnessCfgLoad(strLstSize(argList), strLstPtr(argList));
+        strLstAddZ(argList, "--" CFGOPT_ARCHIVE_ASYNC);
+        harnessCfgLoadRole(cfgCmdArchiveGet, cfgCmdRoleAsync, argList);
 
         String *walSegment = strNew("000000010000000100000001");
 
         TEST_RESULT_VOID(
             archiveAsyncStatusErrorWrite(archiveModeGet, walSegment, 25, strNew("error message")), "write error");
-        TEST_RESULT_STR(
-            strPtr(strNewBuf(storageGetNP(storageNewReadNP(storageTest, strNew("archive/db/in/000000010000000100000001.error"))))),
+        TEST_RESULT_STR_Z(
+            strNewBuf(storageGetP(storageNewReadP(storageTest, strNew("archive/db/in/000000010000000100000001.error")))),
             "25\nerror message", "check error");
         TEST_RESULT_VOID(
             storageRemoveP(storageTest, strNew("archive/db/in/000000010000000100000001.error"), .errorOnMissing = true),
@@ -131,8 +128,8 @@ testRun(void)
 
         TEST_RESULT_VOID(
             archiveAsyncStatusErrorWrite(archiveModeGet, NULL, 25, strNew("global error message")), "write global error");
-        TEST_RESULT_STR(
-            strPtr(strNewBuf(storageGetNP(storageNewReadNP(storageTest, strNew("archive/db/in/global.error"))))),
+        TEST_RESULT_STR_Z(
+            strNewBuf(storageGetP(storageNewReadP(storageTest, strNew("archive/db/in/global.error")))),
             "25\nglobal error message", "check global error");
         TEST_RESULT_VOID(
             storageRemoveP(storageTest, strNew("archive/db/in/global.error"), .errorOnMissing = true),
@@ -140,8 +137,8 @@ testRun(void)
 
         TEST_RESULT_VOID(
             archiveAsyncStatusOkWrite(archiveModeGet, walSegment, NULL), "write ok file");
-        TEST_RESULT_STR(
-            strPtr(strNewBuf(storageGetNP(storageNewReadNP(storageTest, strNew("archive/db/in/000000010000000100000001.ok"))))),
+        TEST_RESULT_STR_Z(
+            strNewBuf(storageGetP(storageNewReadP(storageTest, strNew("archive/db/in/000000010000000100000001.ok")))),
             "", "check ok");
         TEST_RESULT_VOID(
             storageRemoveP(storageTest, strNew("archive/db/in/000000010000000100000001.ok"), .errorOnMissing = true),
@@ -149,8 +146,8 @@ testRun(void)
 
         TEST_RESULT_VOID(
             archiveAsyncStatusOkWrite(archiveModeGet, walSegment, strNew("WARNING")), "write ok file with warning");
-        TEST_RESULT_STR(
-            strPtr(strNewBuf(storageGetNP(storageNewReadNP(storageTest, strNew("archive/db/in/000000010000000100000001.ok"))))),
+        TEST_RESULT_STR_Z(
+            strNewBuf(storageGetP(storageNewReadP(storageTest, strNew("archive/db/in/000000010000000100000001.ok")))),
             "0\nWARNING", "check ok warning");
         TEST_RESULT_VOID(
             storageRemoveP(storageTest, strNew("archive/db/in/000000010000000100000001.ok"), .errorOnMissing = true),
@@ -175,14 +172,37 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("walPath()"))
     {
+        const String *pgPath = storagePathP(storageTest, STRDEF("pg"));
+        storagePathCreateP(storageTest, pgPath);
+
+        TEST_RESULT_STR_Z(walPath(strNew("/absolute/path"), pgPath, strNew("test")), "/absolute/path", "absolute path");
+
+        THROW_ON_SYS_ERROR(chdir(strPtr(pgPath)) != 0, PathMissingError, "unable to chdir()");
         TEST_RESULT_STR(
-            strPtr(walPath(strNew("/absolute/path"), strNew("/pg"), strNew("test"))), "/absolute/path", "absolute path");
+            walPath(strNew("relative/path"), pgPath, strNew("test")), strNewFmt("%s/relative/path", strPtr(pgPath)),
+            "relative path");
+
+
+        const String *pgPathLink = storagePathP(storageTest, STRDEF("pg-link"));
+        THROW_ON_SYS_ERROR_FMT(
+            symlink(strPtr(pgPath), strPtr(pgPathLink)) == -1, FileOpenError,
+            "unable to create symlink '%s' to '%s'", strPtr(pgPath), strPtr(pgPathLink));
+
+        THROW_ON_SYS_ERROR(chdir(strPtr(pgPath)) != 0, PathMissingError, "unable to chdir()");
         TEST_RESULT_STR(
-            strPtr(walPath(strNew("relative/path"), strNew("/pg"), strNew("test"))), "/pg/relative/path", "relative path");
+            walPath(strNew("relative/path"), pgPathLink, strNew("test")), strNewFmt("%s/relative/path", strPtr(pgPathLink)),
+            "relative path");
+
+
+        THROW_ON_SYS_ERROR(chdir("/") != 0, PathMissingError, "unable to chdir()");
+        TEST_ERROR(
+            walPath(strNew("relative/path"), pgPathLink, strNew("test")), AssertError,
+            hrnReplaceKey("working path '/' is not the same path as '{[path]}/pg-link'"));
+
         TEST_ERROR(
             walPath(strNew("relative/path"), NULL, strNew("test")), OptionRequiredError,
             "option 'pg1-path' must be specified when relative wal paths are used\n"
-                "HINT: Is %f passed to test instead of %p?\n"
+                "HINT: is %f passed to test instead of %p?\n"
                 "HINT: PostgreSQL may pass relative paths even with %p depending on the environment.");
     }
 
@@ -191,19 +211,20 @@ testRun(void)
     {
         // Load configuration to set repo-path and stanza
         StringList *argList = strLstNew();
-        strLstAddZ(argList, "pgbackrest");
         strLstAddZ(argList, "--stanza=db");
         strLstAdd(argList, strNewFmt("--repo-path=%s", testPath()));
         strLstAddZ(argList, "archive-get");
-        harnessCfgLoad(strLstSize(argList), strLstPtr(argList));
+        harnessCfgLoad(cfgCmdArchiveGet, argList);
 
         TEST_RESULT_PTR(walSegmentFind(storageRepo(), strNew("9.6-2"), strNew("123456781234567812345678"), 0), NULL, "no path");
 
-        storagePathCreateNP(storageTest, strNew("archive/db/9.6-2/1234567812345678"));
+        storagePathCreateP(storageTest, strNew("archive/db/9.6-2/1234567812345678"));
         TEST_RESULT_PTR(walSegmentFind(storageRepo(), strNew("9.6-2"), strNew("123456781234567812345678"), 0), NULL, "no segment");
-        TEST_RESULT_PTR(
-            walSegmentFind(storageRepo(), strNew("9.6-2"), strNew("123456781234567812345678"), 500), NULL,
-            "no segment after 500ms");
+        TEST_ERROR(
+            walSegmentFind(storageRepo(), strNew("9.6-2"), strNew("123456781234567812345678"), 100), ArchiveTimeoutError,
+            "WAL segment 123456781234567812345678 was not archived before the 100ms timeout\n"
+            "HINT: check the archive_command to ensure that all options are correct (especially --stanza).\n"
+            "HINT: check the PostgreSQL server log for errors.");
 
         // Check timeout by making the wal segment appear after 250ms
         HARNESS_FORK_BEGIN()
@@ -212,8 +233,8 @@ testRun(void)
             {
                 sleepMSec(250);
 
-                storagePutNP(
-                    storageNewWriteNP(
+                storagePutP(
+                    storageNewWriteP(
                         storageTest,
                         strNew(
                             "archive/db/9.6-2/1234567812345678/123456781234567812345678-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa")),
@@ -223,16 +244,16 @@ testRun(void)
 
             HARNESS_FORK_PARENT_BEGIN()
             {
-                TEST_RESULT_STR(
-                    strPtr(walSegmentFind(storageRepo(), strNew("9.6-2"), strNew("123456781234567812345678"), 1000)),
+                TEST_RESULT_STR_Z(
+                    walSegmentFind(storageRepo(), strNew("9.6-2"), strNew("123456781234567812345678"), 1000),
                     "123456781234567812345678-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", "found segment");
             }
             HARNESS_FORK_PARENT_END();
         }
         HARNESS_FORK_END();
 
-        storagePutNP(
-            storageNewWriteNP(
+        storagePutP(
+            storageNewWriteP(
                 storageTest,
                 strNew("archive/db/9.6-2/1234567812345678/123456781234567812345678-bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb.gz")),
             NULL);
@@ -245,7 +266,7 @@ testRun(void)
                 ", 123456781234567812345678-bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb.gz"
                 "\nHINT: are multiple primaries archiving to this stanza?");
 
-        TEST_RESULT_STR(
+        TEST_RESULT_PTR(
             walSegmentFind(storageRepo(), strNew("9.6-2"), strNew("123456781234567812345678.partial"), 0), NULL,
             "did not find partial segment");
     }
@@ -253,47 +274,47 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("walSegmentNext()"))
     {
-        TEST_RESULT_STR(
-            strPtr(walSegmentNext(strNew("000000010000000100000001"), 16 * 1024 * 1024, PG_VERSION_10)),
-            "000000010000000100000002", "get next");
-        TEST_RESULT_STR(
-            strPtr(walSegmentNext(strNew("0000000100000001000000FE"), 16 * 1024 * 1024, PG_VERSION_93)),
-            "0000000100000001000000FF", "get next");
-        TEST_RESULT_STR(
-            strPtr(walSegmentNext(strNew("0000009900000001000000FF"), 16 * 1024 * 1024, PG_VERSION_93)),
-            "000000990000000200000000", "get next overflow >= 9.3");
-        TEST_RESULT_STR(
-            strPtr(walSegmentNext(strNew("0000000100000001000000FE"), 16 * 1024 * 1024, PG_VERSION_92)),
-            "000000010000000200000000", "get next overflow < 9.3");
-        TEST_RESULT_STR(
-            strPtr(walSegmentNext(strNew("000000010000000100000003"), 1024 * 1024 * 1024, PG_VERSION_11)),
-            "000000010000000200000000", "get next overflow >= 11/1GB");
-        TEST_RESULT_STR(
-            strPtr(walSegmentNext(strNew("000000010000006700000FFF"), 1024 * 1024, PG_VERSION_11)),
-            "000000010000006800000000", "get next overflow >= 11/1MB");
+        TEST_RESULT_STR_Z(
+            walSegmentNext(strNew("000000010000000100000001"), 16 * 1024 * 1024, PG_VERSION_10), "000000010000000100000002",
+            "get next");
+        TEST_RESULT_STR_Z(
+            walSegmentNext(strNew("0000000100000001000000FE"), 16 * 1024 * 1024, PG_VERSION_93), "0000000100000001000000FF",
+            "get next");
+        TEST_RESULT_STR_Z(
+            walSegmentNext(strNew("0000009900000001000000FF"), 16 * 1024 * 1024, PG_VERSION_93), "000000990000000200000000",
+            "get next overflow >= 9.3");
+        TEST_RESULT_STR_Z(
+            walSegmentNext(strNew("0000000100000001000000FE"), 16 * 1024 * 1024, PG_VERSION_92), "000000010000000200000000",
+            "get next overflow < 9.3");
+        TEST_RESULT_STR_Z(
+            walSegmentNext(strNew("000000010000000100000003"), 1024 * 1024 * 1024, PG_VERSION_11), "000000010000000200000000",
+            "get next overflow >= 11/1GB");
+        TEST_RESULT_STR_Z(
+            walSegmentNext(strNew("000000010000006700000FFF"), 1024 * 1024, PG_VERSION_11), "000000010000006800000000",
+            "get next overflow >= 11/1MB");
     }
 
     // *****************************************************************************************************************************
     if (testBegin("walSegmentRange()"))
     {
-        TEST_RESULT_STR(
-            strPtr(strLstJoin(walSegmentRange(strNew("000000010000000100000000"), 16 * 1024 * 1024, PG_VERSION_92, 1), "|")),
+        TEST_RESULT_STR_Z(
+            strLstJoin(walSegmentRange(strNew("000000010000000100000000"), 16 * 1024 * 1024, PG_VERSION_92, 1), "|"),
             "000000010000000100000000", "get single");
-        TEST_RESULT_STR(
-            strPtr(strLstJoin(walSegmentRange(strNew("0000000100000001000000FD"), 16 * 1024 * 1024, PG_VERSION_92, 4), "|")),
+        TEST_RESULT_STR_Z(
+            strLstJoin(walSegmentRange(strNew("0000000100000001000000FD"), 16 * 1024 * 1024, PG_VERSION_92, 4), "|"),
             "0000000100000001000000FD|0000000100000001000000FE|000000010000000200000000|000000010000000200000001",
             "get range < 9.3");
-        TEST_RESULT_STR(
-            strPtr(strLstJoin(walSegmentRange(strNew("0000000100000001000000FD"), 16 * 1024 * 1024, PG_VERSION_93, 4), "|")),
+        TEST_RESULT_STR_Z(
+            strLstJoin(walSegmentRange(strNew("0000000100000001000000FD"), 16 * 1024 * 1024, PG_VERSION_93, 4), "|"),
             "0000000100000001000000FD|0000000100000001000000FE|0000000100000001000000FF|000000010000000200000000",
             "get range >= 9.3");
-        TEST_RESULT_STR(
-            strPtr(strLstJoin(walSegmentRange(strNew("000000080000000A00000000"), 1024 * 1024 * 1024, PG_VERSION_11, 8), "|")),
+        TEST_RESULT_STR_Z(
+            strLstJoin(walSegmentRange(strNew("000000080000000A00000000"), 1024 * 1024 * 1024, PG_VERSION_11, 8), "|"),
             "000000080000000A00000000|000000080000000A00000001|000000080000000A00000002|000000080000000A00000003|"
             "000000080000000B00000000|000000080000000B00000001|000000080000000B00000002|000000080000000B00000003",
             "get range >= 11/1GB");
-        TEST_RESULT_STR(
-            strPtr(strLstJoin(walSegmentRange(strNew("000000070000000700000FFE"), 1024 * 1024, PG_VERSION_11, 4), "|")),
+        TEST_RESULT_STR_Z(
+            strLstJoin(walSegmentRange(strNew("000000070000000700000FFE"), 1024 * 1024, PG_VERSION_11, 4), "|"),
             "000000070000000700000FFE|000000070000000700000FFF|000000070000000800000000|000000070000000800000001",
             "get range >= 11/1MB");
     }
